@@ -155,15 +155,9 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
     pixel;
 
   Quantum
-    index;
-
-  Quantum
+    index,
     *q;
 
-  ssize_t
-    i,
-    x;
-
   size_t
     base,
     flag,
@@ -171,7 +165,9 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
   ssize_t
     count,
+    i,
     offset,
+    x,
     y;
 
   TGAInfo
@@ -228,7 +224,7 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
   tga_info.attributes=(unsigned char) ReadBlobByte(image);
   if (EOFBlob(image) != MagickFalse)
     ThrowReaderException(CorruptImageError,"UnableToReadImageData");
-  if ((((tga_info.bits_per_pixel <= 1) || (tga_info.bits_per_pixel >= 17)) &&
+  if ((((tga_info.bits_per_pixel < 1) || (tga_info.bits_per_pixel >= 17)) &&
        (tga_info.bits_per_pixel != 24) && (tga_info.bits_per_pixel != 32)))
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   /*
@@ -449,6 +445,29 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if (skip == MagickFalse)
         switch (tga_info.bits_per_pixel)
         {
+          case 1:
+          {
+            if ((x & 0x07) == 0)
+              {
+                if (ReadBlob(image,1,pixels) != 1)
+                  ThrowReaderException(CorruptImageError,
+                    "UnableToReadImageData");
+                index=(Quantum) pixels[0];
+              }
+            else
+              index=(Quantum) ((size_t) index << 1);
+            if (tga_info.colormap_type != 0)
+              pixel=image->colormap[(ssize_t) ConstrainColormapIndex(image,
+                ((unsigned char) index) & 0x80 ? 1 : 0,exception)];
+            else
+              {
+                pixel.red=(MagickRealType) (((unsigned char) index) & 0x80 ?
+                  QuantumRange : 0);
+                pixel.green=pixel.red;
+                pixel.blue=pixel.red;
+              }
+            break;
+          }
           case 8:
           default:
           {
@@ -465,10 +484,8 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               {
                 pixel.red=(MagickRealType) ScaleCharToQuantum((unsigned char)
                   index);
-                pixel.green=(MagickRealType) ScaleCharToQuantum((unsigned char)
-                  index);
-                pixel.blue=(MagickRealType) ScaleCharToQuantum((unsigned char)
-                  index);
+                pixel.green=pixel.red;
+                pixel.blue=pixel.red;
               }
             break;
           }
@@ -734,19 +751,15 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image,
   QuantumAny
     range;
 
-  ssize_t
-    x;
-
-  ssize_t
-    i;
-
   size_t
     channels;
 
   ssize_t
     base,
     count,
+    i,
     offset,
+    x,
     y;
 
   TGAInfo
